@@ -1,21 +1,25 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2002-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2015 Kern Sibbald
+   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
- *  This file contains all the SQL commands that are either issued by the
- *  Director or which are database backend specific.
+ *  This file contains all the SQL commands that are either issued by
+ *   the Director or which are database backend specific.
  *
  *     Written by Kern Sibbald, July MMII
  */
@@ -28,7 +32,7 @@
 
 #include "bacula.h"
 
-const char *get_restore_objects =
+const char *get_restore_objects = 
    "SELECT JobId,ObjectLength,ObjectFullLength,ObjectIndex,"
            "ObjectType,ObjectCompression,FileIndex,ObjectName,"
            "RestoreObject,PluginName "
@@ -40,28 +44,27 @@ const char *get_restore_objects =
 const char *cleanup_created_job =
    "UPDATE Job SET JobStatus='f', StartTime=SchedTime, EndTime=SchedTime "
    "WHERE JobStatus = 'C'";
-const char *cleanup_running_job =
+const char *cleanup_running_job = 
    "UPDATE Job SET JobStatus='f', EndTime=StartTime WHERE JobStatus = 'R'";
 
 /* For sql_update.c db_update_stats */
 const char *fill_jobhisto =
-        "INSERT INTO JobHisto ("
-           "JobId, Job, Name, Type, Level, ClientId, JobStatus, "
-           "SchedTime, StartTime, EndTime, RealEndTime, JobTDate, "
-           "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes, "
-           "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId, "
-           "PurgedFiles, HasBase, Reviewed, Comment ) "
-        "SELECT "
-           "JobId, Job, Name, Type, Level, ClientId, JobStatus, "
-           "SchedTime, StartTime, EndTime, RealEndTime, JobTDate, "
-           "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes, "
-           "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId, "
+        "INSERT INTO JobHisto (JobId, Job, Name, Type, Level,"
+           "ClientId, JobStatus,"
+           "SchedTime, StartTime, EndTime, RealEndTime, JobTDate,"
+           "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes,"
+           "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId,"
+           "PurgedFiles, HasBase, Reviewed, Comment)"
+        "SELECT JobId, Job, Name, Type, Level, ClientId, JobStatus,"
+           "SchedTime, StartTime, EndTime, RealEndTime, JobTDate,"
+           "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes,"
+           "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId,"
            "PurgedFiles, HasBase, Reviewed, Comment "
           "FROM Job "
-         "WHERE JobStatus IN ('T','W','f','A','E') "
+         "WHERE JobStatus IN ('T','W','f','A','E')"
            "AND NOT EXISTS "
                 "(SELECT JobHisto.JobId "
-                   "FROM JobHisto WHERE JobHisto.Jobid=Job.JobId) "
+                   "FROM JobHisto WHERE JobHisto.Jobid=Job.JobId)"
            "AND JobTDate < %s ";
 
 /* For ua_update.c */
@@ -74,18 +77,18 @@ const char *client_backups =
    " FROM Client,Job,JobMedia,Media,FileSet"
    " WHERE Client.Name='%s'"
    " AND FileSet='%s'"
-   " AND Client.ClientId=Job.ClientId"
-   " AND JobStatus IN ('T','W') AND Type='B'"
-   " AND JobMedia.JobId=Job.JobId AND JobMedia.MediaId=Media.MediaId"
+   " AND Client.ClientId=Job.ClientId "
+   " AND JobStatus IN ('T','W') AND Type='B' "
+   " AND JobMedia.JobId=Job.JobId AND JobMedia.MediaId=Media.MediaId "
    " AND Job.FileSetId=FileSet.FileSetId"
    " ORDER BY Job.StartTime";
 
 /* ====== ua_prune.c */
 
-const char *sel_JobMedia =
-   "SELECT DISTINCT JobMedia.JobId FROM JobMedia,Job "
-   "WHERE MediaId=%s AND Job.JobId=JobMedia.JobId "
-   "AND Job.JobTDate<%s";
+const char *sel_JobMedia = 
+   "SELECT DISTINCT JobMedia.JobId FROM JobMedia,Job"
+   " WHERE MediaId=%s AND Job.JobId=JobMedia.JobId "
+   " AND Job.JobTDate<%s";
 
 /* Delete temp tables and indexes  */
 const char *drop_deltabs[] = {
@@ -105,7 +108,7 @@ const char *uar_list_jobs =
    "FROM Client,Job WHERE Client.ClientId=Job.ClientId AND JobStatus IN ('T','W') "
    "AND Type='B' ORDER BY StartTime DESC LIMIT 20";
 
-const char *uar_print_jobs =
+const char *uar_print_jobs = 
    "SELECT DISTINCT JobId,Level,JobFiles,JobBytes,StartTime,VolumeName"
    " FROM Job JOIN JobMedia USING (JobId) JOIN Media USING (MediaId) "
    " WHERE JobId IN (%s) "
@@ -187,7 +190,7 @@ const char *uar_list_temp =
    " ORDER BY StartTime ASC";
 
 
-const char *uar_sel_jobid_temp =
+const char *uar_sel_jobid_temp = 
    "SELECT DISTINCT JobId,StartTime FROM temp ORDER BY StartTime ASC";
 
 const char *uar_sel_all_temp1 = "SELECT * FROM temp1";
@@ -245,18 +248,18 @@ const char *uar_jobids_fileindex =
    "ORDER BY Job.StartTime DESC LIMIT 1";
 
 /* Query to get list of files from table -- presuably built by an external program */
-const char *uar_jobid_fileindex_from_table =
-   "SELECT JobId,FileIndex FROM %s ORDER BY JobId, FileIndex ASC";
+const char *uar_jobid_fileindex_from_table = 
+   "SELECT JobId, FileIndex FROM %s ORDER BY JobId, FileIndex ASC";
 
-/* Get the list of the last recent version per Delta with a given jobid list
- * This is a tricky part because with SQL the result of
+/* Get the list of the last recent version per Delta with a given
+ *  jobid list. This is a tricky part because with SQL the result of:
  *
- * SELECT MAX(A), B, C, D FROM... GROUP BY (B,C)
+ *   SELECT MAX(A), B, C, D FROM... GROUP BY (B,C)
  *
  * doesn't give the good result (for D).
  *
  * With PostgreSQL, we can use DISTINCT ON(), but with Mysql or Sqlite,
- * we need an extra join using JobTDate.
+ *  we need an extra join using JobTDate.
  */
 static const char *select_recent_version_with_basejob_default =
 "SELECT FileId, Job.JobId AS JobId, FileIndex, File.PathId AS PathId, "
@@ -285,8 +288,9 @@ static const char *select_recent_version_with_basejob_default =
   "AND T1.PathId = File.PathId "
   "AND T1.FilenameId = File.FilenameId";
 
-const char *select_recent_version_with_basejob[] = {
-   /* MySQL */
+const char *select_recent_version_with_basejob[] =
+{
+   /* MySQL  */
    select_recent_version_with_basejob_default,
 
    /* Postgresql */    /* The DISTINCT ON () permits to avoid extra join */
@@ -303,10 +307,10 @@ const char *select_recent_version_with_basejob[] = {
        ") AS T JOIN Job USING (JobId) "
    "ORDER BY FilenameId, PathId, JobTDate DESC ",
 
-   /* SQLite3 */
+   /* SQLite */
    select_recent_version_with_basejob_default
 };
-
+ 
 /* We do the same thing than the previous query, but we include
  * all delta parts. If the file has been deleted, we can have irrelevant
  * parts.
@@ -318,8 +322,7 @@ const char *select_recent_version_with_basejob[] = {
  *
  * If we detect a gap, we can discard further pieces
  * If a file starts at 1 instead of 0, the file has been deleted, and further
- * pieces are useless.
- *
+ *   pieces are useless.
  * This control should be reset for each new file
  */
 static const char *select_recent_version_with_basejob_and_delta_default =
@@ -367,14 +370,14 @@ const char *select_recent_version_with_basejob_and_delta[] = {
        ") AS T JOIN Job USING (JobId) "
    "ORDER BY FilenameId, PathId, DeltaSeq, JobTDate DESC ",
 
-   /* SQLite3 */
+   /* SQLite */
    select_recent_version_with_basejob_and_delta_default
 };
 
 /* Get the list of the last recent version with a given BaseJob jobid list
  * We don't handle Delta with BaseJobs, they have only Full files
  */
-static const char *select_recent_version_default =
+static const char *select_recent_version_default = 
   "SELECT j1.JobId AS JobId, f1.FileId AS FileId, f1.FileIndex AS FileIndex, "
           "f1.PathId AS PathId, f1.FilenameId AS FilenameId, "
           "f1.LStat AS LStat, f1.MD5 AS MD5, j1.JobTDate "
@@ -390,7 +393,8 @@ static const char *select_recent_version_default =
       "AND t1.PathId = f1.PathId "
       "AND j1.JobId = f1.JobId";
 
-const char *select_recent_version[] = {
+const char *select_recent_version[] =
+{
    /* MySQL */
    select_recent_version_default,
 
@@ -401,14 +405,14 @@ const char *select_recent_version[] = {
     "WHERE JobId IN (%s) "
     "ORDER BY FilenameId, PathId, JobTDate DESC ",
 
-   /* SQLite3 */
+   /* SQLite */
    select_recent_version_default
 };
 
-/* We don't create this table as TEMPORARY because MySQL MyISAM
- * 5.0 and 5.1 are unable to run further queries in this mode
+/* We don't create this table as TEMPORARY because MySQL 
+    MyISAM 5.0 and 5.1 are unable to run further queries in this mode
  */
-static const char *create_temp_accurate_jobids_default =
+static const char *create_temp_accurate_jobids_default = 
  "CREATE TABLE btemp3%s AS "
     "SELECT JobId, StartTime, EndTime, JobTDate, PurgedFiles "
       "FROM Job JOIN FileSet USING (FileSetId) "
@@ -482,7 +486,8 @@ const char *create_temp_new_basefile[] = {
 /* ====== ua_prune.c */
 
 /* List of SQL commands to create temp table and indicies  */
-const char *create_deltabs[] = {
+const char *create_deltabs[] =
+{
    /* MySQL */
    "CREATE TEMPORARY TABLE DelCandidates ("
    "JobId INTEGER UNSIGNED NOT NULL, "
@@ -490,29 +495,29 @@ const char *create_deltabs[] = {
    "FileSetId INTEGER UNSIGNED, "
    "JobFiles INTEGER UNSIGNED, "
    "JobStatus BINARY(1))",
-
-   /* Postgresql */
-   "CREATE TEMPORARY TABLE DelCandidates ("
+ 
+   /* PostgreSQL */
+   "CREATE TEMPORARY TABLE DelCandidates ( "
    "JobId INTEGER NOT NULL, "
    "PurgedFiles SMALLINT, "
    "FileSetId INTEGER, "
    "JobFiles INTEGER, "
    "JobStatus char(1))",
-
-   /* SQLite3 */
+ 
+   /* SQLite */
    "CREATE TEMPORARY TABLE DelCandidates ("
    "JobId INTEGER UNSIGNED NOT NULL, "
    "PurgedFiles TINYINT, "
    "FileSetId INTEGER UNSIGNED, "
    "JobFiles INTEGER UNSIGNED, "
    "JobStatus CHAR)"
-};
+}; 
 
-/* ======= ua_purge.c */
-
-/* Select the first available Copy Job that must be upgraded to a Backup job when the original backup job is expired. */
-
-static const char *uap_upgrade_copies_oldest_job_default =
+/* ======= ua_purge.c ====== */
+/* Select the first available Copy Job that must be upgraded 
+ *   to a Backup job when the original backup job is expired.
+ */
+static const char *uap_upgrade_copies_oldest_job_default = 
 "CREATE TEMPORARY TABLE cpy_tmp AS "
        "SELECT MIN(JobId) AS JobId FROM Job "     /* Choose the oldest job */
         "WHERE Type='%c' "                        /* JT_JOB_COPY */
@@ -527,22 +532,22 @@ static const char *uap_upgrade_copies_oldest_job_default =
               ") "
           "GROUP BY PriorJobId ";           /* one result per copy */
 
-const char *uap_upgrade_copies_oldest_job[] = {
-   /* Mysql */
+const char *uap_upgrade_copies_oldest_job[] =
+{
+   /* MySQL */
    uap_upgrade_copies_oldest_job_default,
-
-   /* Postgresql */
+   /* PostgreSQL */
    uap_upgrade_copies_oldest_job_default,
-
-   /* SQLite3 */
+   /* SQLite */
    uap_upgrade_copies_oldest_job_default
-};
-
-/* ======= ua_restore.c */
+}; 
+ 
+/* ======= ua_restore.c ====== */
 
 /* List Jobs where a particular file is saved */
-const char *uar_file[] = {
-   /* Mysql */
+const char *uar_file[] =
+{
+   /* MySQL */
    "SELECT Job.JobId as JobId,"
    "CONCAT(Path.Path,Filename.Name) as Name, "
    "StartTime,Type as JobType,JobStatus,JobFiles,JobBytes "
@@ -587,8 +592,8 @@ const char *uar_create_temp[] = {
    "StartFile INTEGER UNSIGNED,"
    "VolSessionId INTEGER UNSIGNED,"
    "VolSessionTime INTEGER UNSIGNED)",
-
-   /* Postgresql */
+ 
+   /* PostgreSQL */
    "CREATE TEMPORARY TABLE temp ("
    "JobId INTEGER NOT NULL,"
    "JobTDate BIGINT,"
@@ -601,8 +606,8 @@ const char *uar_create_temp[] = {
    "StartFile INTEGER,"
    "VolSessionId INTEGER,"
    "VolSessionTime INTEGER)",
-
-   /* SQLite3 */
+ 
+   /* SQLite */
    "CREATE TEMPORARY TABLE temp ("
    "JobId INTEGER UNSIGNED NOT NULL,"
    "JobTDate BIGINT UNSIGNED,"
@@ -615,26 +620,25 @@ const char *uar_create_temp[] = {
    "StartFile INTEGER UNSIGNED,"
    "VolSessionId INTEGER UNSIGNED,"
    "VolSessionTime INTEGER UNSIGNED)"
-};
+}; 
 
-const char *uar_create_temp1[] = {
-   /* Mysql */
+const char *uar_create_temp1[] =
+{
+   /* MySQL */
    "CREATE TEMPORARY TABLE temp1 ("
    "JobId INTEGER UNSIGNED NOT NULL,"
    "JobTDate BIGINT UNSIGNED)",
-
-   /* Postgresql */
+   /* PostgreSQL */
    "CREATE TEMPORARY TABLE temp1 ("
    "JobId INTEGER NOT NULL,"
    "JobTDate BIGINT)",
-
-   /* SQLite3 */
+   /* SQLite */
    "CREATE TEMPORARY TABLE temp1 ("
    "JobId INTEGER UNSIGNED NOT NULL,"
    "JobTDate BIGINT UNSIGNED)"
-};
+}; 
 
-/* Query to get all files in a directory -- no recursing
+/* Query to get all files in a directory no recursing
  *  Note, for PostgreSQL since it respects the "Single Value
  *  rule", the results of the SELECT will be unoptimized.
  *  I.e. the same file will be restored multiple times, once
@@ -689,12 +693,10 @@ const char *sql_media_order_most_recently_written[] = {
 const char *sql_get_max_connections[] = {
    /* Mysql */
    "SHOW VARIABLES LIKE 'max_connections'",
-
-   /* Postgresql */
+   /* PostgreSQL */
    "SHOW max_connections",
-
-   /* SQLite3 */
-   "SELECT 0"
+   /* SQLite */
+   "SELECT  0"
 };
 
 /*
@@ -715,11 +717,11 @@ const char *default_sql_bvfs_select =
   "AND File.FileIndex > 0 "
   "AND Job.JobId IN (SELECT DISTINCT JobId FROM btemp%s) ";
 
-const char *sql_bvfs_select[] = {
-   /* Mysql */
+const char *sql_bvfs_select[] =
+{
+   /* MySQL */
    default_sql_bvfs_select,
-
-   /* Postgresql */
+   /* PostgreSQL */
    "CREATE TABLE %s AS ( "
         "SELECT JobId, FileIndex, FileId "
           "FROM ( "
@@ -729,8 +731,7 @@ const char *sql_bvfs_select[] = {
               "ORDER BY PathId, FilenameId, JobTDate DESC "
           ") AS T "
           "WHERE FileIndex > 0)",
-
-   /* SQLite3 */
+   /* SQLite */
    default_sql_bvfs_select
 };
 
@@ -764,8 +765,7 @@ static const char *sql_bvfs_list_files_default =
        "OR Job.JobId IN (%s)) ";
 
 const char *sql_bvfs_list_files[] = {
-   /* Mysql */
-   /* JobId PathId JobId PathId Limit Offset AND? Filename? JobId JobId*/
+   /* MySQL */
    sql_bvfs_list_files_default,
 
    /* JobId PathId JobId PathId WHERE? Filename? Limit Offset*/
@@ -792,13 +792,13 @@ const char *sql_bvfs_list_files[] = {
    /* SQLite */
    sql_bvfs_list_files_default,
 
-   /* SQLite3 */
+   /* SQLite */
    sql_bvfs_list_files_default
 };
 
 /* Basically the same thing than select_recent_version_with_basejob_and_delta_default,
  * but we specify a single file with FilenameId/PathId
- *
+ * 
  * Input:
  * 1 JobId to look at
  * 2 FilenameId
@@ -820,7 +820,7 @@ const char *bvfs_select_delta_version_with_basejob_and_delta_default =
          "FROM File JOIN Job USING (JobId) "          /* from selected backup */
         "WHERE File.JobId IN (%s) AND FilenameId = %s AND PathId = %s "
          "UNION ALL "
-       "SELECT JobTDate, PathId, FilenameId, DeltaSeq " /*Get all files from */
+       "SELECT JobTDate, PathId, FilenameId, DeltaSeq " /*Get all files from */ 
          "FROM BaseFiles "                            /* BaseJob */
               "JOIN File USING (FileId) "
               "JOIN Job  ON    (BaseJobId = Job.JobId) "
@@ -838,7 +838,8 @@ const char *bvfs_select_delta_version_with_basejob_and_delta_default =
   "AND T1.FilenameId = File.FilenameId";
 
 
-const char *bvfs_select_delta_version_with_basejob_and_delta[] = {
+const char *bvfs_select_delta_version_with_basejob_and_delta[] =
+{
    /* MySQL */
    bvfs_select_delta_version_with_basejob_and_delta_default,
 
@@ -856,7 +857,7 @@ const char *bvfs_select_delta_version_with_basejob_and_delta[] = {
        ") AS T JOIN Job USING (JobId) "
    "ORDER BY FilenameId, PathId, DeltaSeq, JobTDate DESC ",
 
-   /* SQLite3 */
+   /* SQLite */
    bvfs_select_delta_version_with_basejob_and_delta_default
 };
 
@@ -900,15 +901,15 @@ const char *batch_fill_path_query[] = {
       "SELECT a.Path FROM "
          "(SELECT DISTINCT Path FROM batch) AS a WHERE NOT EXISTS "
          "(SELECT Path FROM Path AS p WHERE p.Path = a.Path)",
-
-   /* Postgresql */
-   "INSERT INTO Path (Path) "
+ 
+   /* PostgreSQL */
+   "INSERT INTO Path (Path)"
       "SELECT a.Path FROM "
          "(SELECT DISTINCT Path FROM batch) AS a "
        "WHERE NOT EXISTS (SELECT Path FROM Path WHERE Path = a.Path) ",
-
-   /* SQLite3 */
-   "INSERT INTO Path (Path) "
+ 
+   /* SQLite */
+   "INSERT INTO Path (Path)"
       "SELECT DISTINCT Path FROM batch "
       "EXCEPT SELECT Path FROM Path"
 };
@@ -936,85 +937,88 @@ const char *batch_fill_filename_query[] = {
 const char *match_query[] = {
    /* Mysql */
    "REGEXP",
-
-   /* Postgresql */
-   "~",
-
-   /* SQLite3 */
-   "LIKE"                      /* MATCH doesn't seems to work anymore... */
-};
-
+   /* PostgreSQL */
+   "~", 
+   /* SQLite */
+   "LIKE"                       /* MATCH doesn't seems to work anymore... */
+}; 
+ 
 static const char *insert_counter_values_default =
    "INSERT INTO Counters (Counter, MinValue, "
    "MaxValue, CurrentValue, WrapCounter) "
    "VALUES ('%s','%d','%d','%d','%s')";
 
 const char *insert_counter_values[] = {
-   /* Mysql */
+   /* MySQL */
    "INSERT INTO Counters (Counter, Counters.MinValue, "
    "Counters.MaxValue, CurrentValue, WrapCounter) "
    "VALUES ('%s','%d','%d','%d','%s')",
 
-   /* Postgresql */
+   /* PostgreSQL */
    insert_counter_values_default,
 
-   /* SQLite3 */
+   /* SQLite */
    insert_counter_values_default
 };
 
-static const char *select_counter_values_default =
-   "SELECT MinValue, MaxValue, CurrentValue, WrapCounter "
-   "FROM Counters WHERE Counter='%s'";
-
-const char *select_counter_values[] = {
-   /* Mysql */
-   "SELECT Counters.MinValue, Counters.MaxValue, CurrentValue, WrapCounter "
-   "FROM Counters WHERE Counter='%s'",
-
-   /* Postgresql */
-   select_counter_values_default,
-
-   /* SQLite3 */
-   select_counter_values_default
-};
-
-static const char *update_counter_values_default =
-   "UPDATE Counters SET "
-   "MinValue=%d, MaxValue=%d, CurrentValue=%d, WrapCounter='%s'"
-   "WHERE Counter='%s'";
-
-const char *update_counter_values[] = {
-   /* Mysql */
-   "UPDATE Counters SET "
-   "Counters.MinValue=%d, Counters.MaxValue=%d, CurrentValue=%d, WrapCounter='%s'"
-   "WHERE Counter='%s'",
-
-   /* Postgresql */
-   update_counter_values_default,
-
-   /* SQLite3 */
-   update_counter_values_default
-};
-
+static const char *select_counter_values_default = 
+   "SELECT MinValue, MaxValue, CurrentValue, WrapCounter"
+   " FROM Counters WHERE Counter='%s'";
+ 
+const char *select_counter_values[] =
+{
+   /* MySQL */
+   "SELECT Counters.MinValue, Counters.MaxValue, CurrentValue, WrapCounter"
+   " FROM Counters WHERE Counter='%s'",
+ 
+   /* PostgreSQL */
+   select_counter_values_default, 
+ 
+   /* SQLite */
+   select_counter_values_default 
+}; 
+ 
+static const char *update_counter_values_default = 
+   "UPDATE Counters SET MinValue=%d, MaxValue=%d, CurrentValue=%d,"
+    "WrapCounter='%s' WHERE Counter='%s'";
+ 
+const char *update_counter_values[] =
+{
+   /* MySQL */
+   "UPDATE Counters SET Counters.MinValue=%d, Counters.MaxValue=%d,"
+     "CurrentValue=%d, WrapCounter='%s' WHERE Counter='%s'",
+   /* PostgreSQL */
+   update_counter_values_default, 
+   /* SQLite */
+   update_counter_values_default 
+}; 
+ 
 static const char *expired_volumes_defaults =
-"SELECT Media.VolumeName  AS volumename, "
-       "Media.LastWritten AS lastwritten "
-" FROM  Media "
-" WHERE VolStatus IN ('Full', 'Used') "
-     " AND ( Media.LastWritten +  Media.VolRetention ) < NOW() "
+"SELECT Media.VolumeName  AS volumename,"
+       "Media.LastWritten AS lastwritten"
+" FROM  Media"
+" WHERE VolStatus IN ('Full', 'Used')"
+     " AND ( Media.LastWritten +  Media.VolRetention ) < NOW()"
      " %s ";
 
 const char *expired_volumes[] = {
-   /* Mysql */
+   /* MySQL */
    expired_volumes_defaults,
-
-   /* Postgresql */
+   /* PostgreSQL */
    "SELECT Media.VolumeName, Media.LastWritten "
    " FROM  Media "
    " WHERE VolStatus IN ('Full', 'Used') "
      " AND ( Media.LastWritten + (interval '1 second' * Media.VolRetention ) < NOW()) "
      " %s ",
-
-   /* SQLite3 */
+   /* SQLite */
    expired_volumes_defaults
 };
+
+const char *expires_in[] = {
+   /* MySQL */
+   "(GREATEST(0, CAST(UNIX_TIMESTAMP(LastWritten) + VolRetention AS SIGNED) - UNIX_TIMESTAMP(NOW())))",
+   /* PostgreSQL */
+   "GREATEST(0, (extract('epoch' from LastWritten + VolRetention * interval '1second' - NOW())::bigint))",
+   /* SQLite */
+   "MAX(0, (strftime('%s', LastWritten) + VolRetention - strftime('%s', datetime('now', 'localtime'))))"
+};
