@@ -1,24 +1,29 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 1981-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
-
 /*
-  Generalized console input/output handler
-  A maintanable replacement for readline()
+      Generalized console input/output handler
+      A maintanable replacement for readline()
 
-  Written in 1981, updated for Bacula, Kern Sibbald, December MMIII
+         Updated for Bacula, Kern Sibbald, December MMIII
+
+      This code is in part derived from code that I wrote in
+      1981, so some of it is a bit old and could use a cleanup.
 
 */
 
@@ -31,6 +36,7 @@
  *  Each subsequent character starts with 10 (i.e. (c & 0xC0) == 0x80)
  */
 
+
 #ifdef  TEST_PROGRAM
 #include <stdio.h>
 #include <unistd.h>
@@ -51,8 +57,8 @@
 #include <curses.h>
 #include <term.h>
 
-#if defined(HAVE_SUN_OS)
-#if !defined(_TERM_H)
+#ifdef HAVE_SUN_OS
+#ifndef _TERM_H
 extern "C" int tgetent(void *, const char *);
 extern "C" int tgetnum(const char *);
 extern "C" char *tgetstr (const char*, char**);
