@@ -69,7 +69,12 @@ public:
    /* Methods */
    run_ctx() { memset(this, 0, sizeof(run_ctx));
                store = new USTORE; };
-   ~run_ctx() { delete store; };
+   ~run_ctx() {
+      delete store;
+      if (JobIds) {
+         delete JobIds;
+      }
+   };
 };
 
 /* Forward referenced subroutines */
@@ -144,7 +149,6 @@ int run_cmd(UAContext *ua, const char *cmd)
          break; /* error get out of while loop */
       }
 
-
       /* Run without prompting? */
       if (ua->batch || find_arg(ua, NT_("yes")) > 0) {
          return start_job(ua, jcr, rc);
@@ -1067,6 +1071,11 @@ static bool set_run_context_in_jcr(UAContext *ua, JCR *jcr, run_ctx &rc)
       jcr->IgnoreDuplicateJobChecking = rc.ignoreduplicatecheck;
    }
 
+   /* Not a good idea to start a job with the Scratch pool */
+   if (rc.pool && strcmp(rc.pool->name(), NT_("Scratch")) == 0) {
+      ua->send_msg(_("Pool \"Scratch\" not valid.\n"));
+      return false;
+   }
    return true;
 }
 
