@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2015 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -54,6 +57,8 @@ int update_cmd(UAContext *ua, const char *cmd)
       NT_("slot"),   /* 4 */
       NT_("jobid"),  /* 5 */
       NT_("stats"),  /* 6 */
+      NT_("snap"),   /* 7 */
+      NT_("snapshot"),/* 8 */
       NULL};
 
    if (!open_client_db(ua)) {
@@ -78,6 +83,10 @@ int update_cmd(UAContext *ua, const char *cmd)
    case 6:
       update_stats(ua);
       return 1;
+   case 7:
+   case 8:
+      update_snapshot(ua);
+      return 1;
    default:
       break;
    }
@@ -87,6 +96,7 @@ int update_cmd(UAContext *ua, const char *cmd)
    add_prompt(ua, _("Pool from resource"));
    add_prompt(ua, _("Slots from autochanger"));
    add_prompt(ua, _("Long term statistics"));
+   add_prompt(ua, _("Snapshot parameters"));
    switch (do_prompt(ua, _("item"), _("Choose catalog item to update"), NULL, 0)) {
    case 0:
       update_volume(ua);
@@ -100,6 +110,9 @@ int update_cmd(UAContext *ua, const char *cmd)
    case 3:
       update_stats(ua);
       break;
+   case 4:
+      update_snapshot(ua);
+      break;
    default:
       break;
    }
@@ -278,9 +291,8 @@ static void update_volslot(UAContext *ua, char *val, MEDIA_DBR *mr)
       return;
    }
    mr->Slot = atoi(val);
-   if (pr.MaxVols > 0 && mr->Slot > (int)pr.MaxVols) {
-      ua->error_msg(_("Invalid slot, it must be between 0 and MaxVols=%d\n"),
-         pr.MaxVols);
+   if (mr->Slot < 0) {
+      ua->error_msg(_("Invalid slot, it must be greater than zero\n"));
       return;
    }
    /*
@@ -508,22 +520,31 @@ static int update_volume(UAContext *ua)
       NT_("InChanger"),                /* 7 */
       NT_("Slot"),                     /* 8 */
       NT_("Pool"),                     /* 9 */
-      NT_("FromPool"),                 /* 10 */
+      NT_("FromPool"),                 /* 10 !!! see below !!! */
       NT_("AllFromPool"),              /* 11 !!! see below !!! */
       NT_("Enabled"),                  /* 12 */
       NT_("RecyclePool"),              /* 13 */
       NT_("ActionOnPurge"),            /* 14 */
+      NT_("FromAllPools"),             /* 15 !!! see bellow !!! */
       NULL };
 
-#define AllFromPool 11               /* keep this updated with above */
+#define FromPool     10              /* keep this updated */
+#define AllFromPool  11              /* keep this updated with above */
+#define FromAllPools 15              /* keep this updated */
 
    for (i=0; kw[i]; i++) {
       int j;
       POOL_DBR pr;
+      /* No argv with these parameters */
+      if (i == FromPool || i == FromAllPools) {
+         j = find_arg(ua, kw[i]);
 
-      if ((j=find_arg_with_value(ua, kw[i])) > 0) {
-         /* If all from pool don't select a media record */
-         if (i != AllFromPool && !select_media_dbr(ua, &mr)) {
+      } else {
+         j = find_arg_with_value(ua, kw[i]);
+      }
+      if (j > 0) {
+         /* If all from pool/from all pools don't select a media record */
+         if (i != AllFromPool && i != FromAllPools && !select_media_dbr(ua, &mr)) {
             return 0;
          }
          switch (i) {
@@ -578,17 +599,14 @@ static int update_volume(UAContext *ua)
          case 14:
             update_vol_actiononpurge(ua, ua->argv[j], &mr);
             break;
+         case 15:
+            update_all_vols(ua);
+            break;
          }
          done = true;
       }
    }
 
-   /* Allow user to simply update all volumes */
-   if (find_arg(ua, NT_("fromallpools")) > 0) {
-      update_all_vols(ua);
-      return 1;
-   }
-
    for ( ; !done; ) {
       start_prompt(ua, _("Parameters to modify:\n"));
       add_prompt(ua, _("Volume Status"));              /* 0 */
