@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2015 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -62,11 +65,10 @@ bool do_verify_init(JCR *jcr)
       free_rstorage(jcr);
       free_wstorage(jcr);
       break;
+   case L_VERIFY_DATA:
    case L_VERIFY_VOLUME_TO_CATALOG:
       free_wstorage(jcr);
       break;
-   case L_VERIFY_DATA:
-      break;
    default:
       Jmsg2(jcr, M_FATAL, 0, _("Unimplemented Verify level %d(%c)\n"), jcr->getJobLevel(),
          jcr->getJobLevel());
@@ -112,11 +114,13 @@ bool do_verify(JCR *jcr)
     */
    if (jcr->getJobLevel() == L_VERIFY_CATALOG ||
        jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG ||
-       jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG) {
+       jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG ||
+       jcr->getJobLevel() == L_VERIFY_DATA) {
       memcpy(&jr, &jcr->jr, sizeof(jr));
       if (jcr->verify_job &&
           (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG ||
-           jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG)) {
+           jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG   ||
+           jcr->getJobLevel() == L_VERIFY_DATA)) {
          Name = jcr->verify_job->name();
       } else {
          Name = NULL;
@@ -151,7 +155,8 @@ bool do_verify(JCR *jcr)
     */
    if (jcr->getJobLevel() == L_VERIFY_CATALOG ||
        jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG ||
-       jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG) {
+       jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG ||
+       jcr->getJobLevel() == L_VERIFY_DATA) {
       jcr->previous_jr.JobId = verify_jobid;
       if (!db_get_job_record(jcr, jcr->db, &jcr->previous_jr)) {
          Jmsg(jcr, M_FATAL, 0, _("Could not get job record for previous Job. ERR=%s"),
@@ -174,7 +179,7 @@ bool do_verify(JCR *jcr)
     *   create a dummy authorization key (passed to
     *   File daemon but not used).
     */
-   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG) {
+   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG || jcr->getJobLevel() == L_VERIFY_DATA) {
       int stat;
       /*
        * Note: negative status is an error, zero status, means
@@ -192,10 +197,43 @@ bool do_verify(JCR *jcr)
       jcr->sd_auth_key = bstrdup("dummy");    /* dummy Storage daemon key */
    }
 
+   /* Pass the original fileset to the client */
+   if (jcr->getJobLevel() == L_VERIFY_DATA) {
+      FILESET_DBR fdbr;
+      memset(&fdbr, 0, sizeof(fdbr));
+      fdbr.FileSetId = jcr->previous_jr.FileSetId;
+      if (!db_get_fileset_record(jcr, jcr->db, &fdbr)) {
+         Jmsg(jcr, M_FATAL, 0,
+              _("Could not get fileset record from previous Job. ERR=%s"),
+              db_strerror(jcr->db));
+         return false;
+      }
+
+      jcr->fileset = (FILESET *)GetResWithName(R_FILESET, fdbr.FileSet);
+      if (!jcr->fileset) {
+         if (jcr->verify_job) {
+            jcr->fileset = jcr->verify_job->fileset;
+            Jmsg(jcr, M_WARNING, 0,
+                 _("Could not find FileSet resource \"%s\" from previous Job\n"),
+                 fdbr.FileSet);
+            Jmsg(jcr, M_INFO, 0,
+                 _("Using FileSet \"%\"\n"), jcr->fileset->name());
+
+         } else {
+            Jmsg(jcr, M_FATAL, 0,
+                 _("Could not get FileSet resource for verify Job."));
+            return false;
+         }
+      }
+      Dmsg1(50, "FileSet = %s\n", jcr->fileset->name());
+   }
+
+   /* Pass the current fileset to the client */
    if (jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG && jcr->verify_job) {
       jcr->fileset = jcr->verify_job->fileset;
    }
-   Dmsg2(100, "ClientId=%u JobLevel=%c\n", jcr->previous_jr.ClientId, jcr->getJobLevel());
+   Dmsg2(100, "ClientId=%u JobLevel=%c\n",
+         jcr->previous_jr.ClientId, jcr->getJobLevel());
 
    if (!db_update_job_start_record(jcr, jcr->db, &jcr->jr)) {
       Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
@@ -206,7 +244,9 @@ bool do_verify(JCR *jcr)
    Jmsg(jcr, M_INFO, 0, _("Start Verify JobId=%s Level=%s Job=%s\n"),
       edit_uint64(jcr->JobId, ed1), level_to_str(jcr->getJobLevel()), jcr->Job);
 
-   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG) {
+   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG ||
+       jcr->getJobLevel() == L_VERIFY_DATA)
+   {
       /*
        * Start conversation with Storage daemon
        */
@@ -269,9 +309,10 @@ bool do_verify(JCR *jcr)
    case L_VERIFY_CATALOG:
       level = "catalog";
       break;
+   case L_VERIFY_DATA:
    case L_VERIFY_VOLUME_TO_CATALOG:
       if (jcr->sd_calls_client) {
-         if (jcr->FDVersion < 5) {
+         if (jcr->FDVersion < 10) {
             Jmsg(jcr, M_FATAL, 0, _("The File daemon does not support SDCallsClient.\n"));
             goto bail_out;
          }
@@ -305,18 +346,19 @@ bool do_verify(JCR *jcr)
          Jmsg0(jcr, M_FATAL, 0, _("Deprecated feature ... use bootstrap.\n"));
          goto bail_out;
       }
-
-      level = "volume";
-      break;
-   case L_VERIFY_DATA:
-      level = "data";
+      if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG) {
+         level = "volume";
+      } else {
+         level = "data";
+      }
       break;
    case L_VERIFY_DISK_TO_CATALOG:
       level="disk_to_catalog";
       break;
    default:
-      Jmsg2(jcr, M_FATAL, 0, _("Unimplemented Verify level %d(%c)\n"), jcr->getJobLevel(),
-         jcr->getJobLevel());
+      Jmsg2(jcr, M_FATAL, 0, _("Unimplemented Verify level %d(%c)\n"),
+            jcr->getJobLevel(),
+            jcr->getJobLevel());
       goto bail_out;
    }
 
@@ -368,6 +410,10 @@ bool do_verify(JCR *jcr)
       db_write_batch_file_records(jcr);
       break;
 
+   case L_VERIFY_DATA:
+      /* Nothing special to do */
+      bget_dirmsg(fd);          /* eat EOD */
+      break;
    default:
       Jmsg1(jcr, M_FATAL, 0, _("Unimplemented verify level %d\n"), jcr->getJobLevel());
       goto bail_out;
@@ -399,7 +445,7 @@ void verify_cleanup(JCR *jcr, int TermCode)
 
    Dmsg3(900, "JobLevel=%c Expected=%u JobFiles=%u\n", jcr->getJobLevel(),
       jcr->ExpectedFiles, jcr->JobFiles);
-   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG &&
+   if ((jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG || jcr->getJobLevel() == L_VERIFY_DATA) &&
        jcr->ExpectedFiles != jcr->JobFiles) {
       TermCode = JS_ErrorTerminated;
    }
@@ -418,7 +464,11 @@ void verify_cleanup(JCR *jcr, int TermCode)
    msg_type = M_INFO;                 /* by default INFO message */
    switch (TermCode) {
    case JS_Terminated:
-      term_msg = _("Verify OK");
+      if (jcr->JobErrors || jcr->SDErrors) {
+         term_msg = _("Verify OK -- with warnings");
+      } else {
+         term_msg = _("Verify OK");
+      }
       break;
    case JS_FatalError:
    case JS_ErrorTerminated:
@@ -449,7 +499,7 @@ void verify_cleanup(JCR *jcr, int TermCode)
    }
 
    jobstatus_to_ascii(jcr->FDJobStatus, fd_term_msg, sizeof(fd_term_msg));
-   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG) {
+   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG || jcr->getJobLevel() == L_VERIFY_DATA) {
       jobstatus_to_ascii(jcr->SDJobStatus, sd_term_msg, sizeof(sd_term_msg));
       Jmsg(jcr, msg_type, 0, _("%s %s %s (%s):\n"
 "  Build OS:               %s %s %s\n"
@@ -465,6 +515,7 @@ void verify_cleanup(JCR *jcr, int TermCode)
 "  Files Expected:         %s\n"
 "  Files Examined:         %s\n"
 "  Non-fatal FD errors:    %d\n"
+"  SD Errors:              %d\n"
 "  FD termination status:  %s\n"
 "  SD termination status:  %s\n"
 "  Termination:            %s\n\n"),
@@ -482,6 +533,7 @@ void verify_cleanup(JCR *jcr, int TermCode)
            edit_uint64_with_commas(jcr->ExpectedFiles, ec1),
            edit_uint64_with_commas(jcr->JobFiles, ec2),
            jcr->JobErrors,
+           jcr->SDErrors,
            fd_term_msg,
            sd_term_msg,
            term_msg);
@@ -552,22 +604,27 @@ void get_attributes_and_compare_to_catalog(JCR *jcr, JobId_t JobId)
     *   Link name  ???
     */
    while ((n=bget_dirmsg(fd)) >= 0 && !job_canceled(jcr)) {
-      int stream;
+      int32_t stream, full_stream;
       char *attr, *p, *fn;
       char Opts_Digest[MAXSTRING];        /* Verify Opts or MD5/SHA1 digest */
 
       if (job_canceled(jcr)) {
-         goto bail_out;
+         free_pool_memory(fname);
+         return;
       }
       fname = check_pool_memory_size(fname, fd->msglen);
       jcr->fname = check_pool_memory_size(jcr->fname, fd->msglen);
       Dmsg1(200, "Atts+Digest=%s\n", fd->msg);
-      if ((len = sscanf(fd->msg, "%ld %d %100s", &file_index, &stream,
+      if ((len = sscanf(fd->msg, "%ld %d %100s", &file_index, &full_stream,
             fname)) != 3) {
          Jmsg3(jcr, M_FATAL, 0, _("bird<filed: bad attributes, expected 3 fields got %d\n"
 " mslen=%d msg=%s\n"), len, fd->msglen, fd->msg);
-         goto bail_out;
+         free_pool_memory(fname);
+         return;
       }
+      stream = full_stream & STREAMMASK_TYPE;
+      Dmsg4(30, "Got hdr: FilInx=%d FullStream=%d Stream=%d fname=%s.\n", file_index, full_stream, stream, fname);
+
       /*
        * We read the Options or Signature into fname
        *  to prevent overrun, now copy it to proper location.
@@ -737,7 +794,8 @@ void get_attributes_and_compare_to_catalog(JCR *jcr, JobId_t JobId)
          if (jcr->FileIndex != (uint32_t)file_index) {
             Jmsg2(jcr, M_FATAL, 0, _("MD5/SHA1 index %d not same as attributes %d\n"),
                file_index, jcr->FileIndex);
-            goto bail_out;
+            free_pool_memory(fname);
+            return;
          }
          if (do_Digest != CRYPTO_DIGEST_NONE) {
             db_escape_string(jcr, jcr->db, buf, Opts_Digest, strlen(Opts_Digest));
@@ -756,7 +814,8 @@ void get_attributes_and_compare_to_catalog(JCR *jcr, JobId_t JobId)
       berrno be;
       Jmsg2(jcr, M_FATAL, 0, _("bdird<filed: bad attributes from filed n=%d : %s\n"),
                         n, be.bstrerror());
-      goto bail_out;
+      free_pool_memory(fname);
+      return;
    }
 
    /* Now find all the files that are missing -- i.e. all files in
@@ -774,8 +833,6 @@ void get_attributes_and_compare_to_catalog(JCR *jcr, JobId_t JobId)
    if (jcr->fn_printed) {
       jcr->setJobStatus(JS_Differences);
    }
-
-bail_out:
    free_pool_memory(fname);
 }
 
