@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -232,7 +235,7 @@ int read_dev_volume_label(DCR *dcr)
          Mmsg3(jcr->errmsg, _("Could not reserve volume %s on %s device %s\n"),
               dev->VolHdr.VolumeName, dev->print_type(), dev->print_name());
       }
-      Dmsg2(150, "Could not reserve volume %s on %s\n", dev->VolHdr.VolumeName, dev->print_name());
+      Dmsg2(100, "Could not reserve volume %s on %s\n", dev->VolHdr.VolumeName, dev->print_name());
       stat = VOL_NAME_ERROR;
       goto bail_out;
    }
@@ -334,6 +337,9 @@ bool write_new_volume_label_to_dev(DCR *dcr, const char *VolName,
 
    Dmsg0(150, "write_volume_label()\n");
    if (*VolName == 0) {
+      if (dcr->jcr) {
+         Mmsg(dcr->jcr->errmsg, "ERROR: new_volume_label_to_dev called with NULL VolName\n");
+      }
       Pmsg0(0, "=== ERROR: write_new_volume_label_to_dev called with NULL VolName\n");
       goto bail_out;
    }
@@ -481,7 +487,7 @@ bool DCR::rewrite_volume_label(bool recycle)
    DCR *dcr = this;
 
    Enter(100);
-   ASSERT(dcr->VolumeName[0]);
+   ASSERT2(dcr->VolumeName[0], "Empty Volume name");
    if (!dev->open(dcr, OPEN_READ_WRITE)) {
        Jmsg4(jcr, M_WARNING, 0, _("Open %s device %s Volume \"%s\" failed: ERR=%s\n"),
              dev->print_type(), dev->print_name(), dcr->VolumeName, dev->bstrerror());
@@ -527,7 +533,7 @@ bool DCR::rewrite_volume_label(bool recycle)
    }
    Dmsg1(100, "wrote vol label to block. Vol=%s\n", dcr->VolumeName);
 
-   ASSERT(dcr->VolumeName[0]);
+   ASSERT2(dcr->VolumeName[0], "Empty Volume name");
    dev->setVolCatInfo(false);
 
    /*
@@ -564,6 +570,12 @@ bool DCR::rewrite_volume_label(bool recycle)
          return false;
       }
    }
+   ASSERT2(dcr->VolumeName[0], "Empty Volume name");
+   dev->setVolCatName(dcr->VolumeName);
+   if (!dir_get_volume_info(dcr, GET_VOL_INFO_FOR_WRITE)) {
+      Leave(100);
+      return false;
+   }
    dev->set_labeled();
    /* Set or reset Volume statistics */
    dev->VolCatInfo.VolCatJobs = 0;
@@ -584,9 +596,7 @@ bool DCR::rewrite_volume_label(bool recycle)
    Dmsg1(100, "dir_update_vol_info. Set Append vol=%s\n", dcr->VolumeName);
    dev->VolCatInfo.VolFirstWritten = time(NULL);
    dev->setVolCatStatus("Append");
-   ASSERT(dcr->VolumeName[0]);
-   dev->setVolCatName(dcr->VolumeName);
-   if (!dir_update_volume_info(dcr, true, true)) {  /* indicate doing relabel */
+   if (!dir_update_volume_info(dcr, true, true)) {  /* indicate relabel */
       Leave(100);
       return false;
    }
@@ -601,7 +611,8 @@ bool DCR::rewrite_volume_label(bool recycle)
     * End writing real Volume label (from pre-labeled tape), or recycling
     *  the volume.
     */
-   Dmsg1(100, "OK from rewrite vol label. Vol=%s\n", dcr->VolumeName);
+   Dmsg3(100, "OK from rewrite vol label. adata=%d slot=%d Vol=%s\n",
+      dcr->block->adata, dev->VolCatInfo.Slot, dcr->VolumeName);
    Leave(100);
    return true;
 }
@@ -661,7 +672,7 @@ static void create_volume_label_record(DCR *dcr, DEVICE *dev,
 
    ser_end(rec->data, SER_LENGTH_Volume_Label);
    bstrncpy(dcr->VolumeName, dev->VolHdr.VolumeName, sizeof(dcr->VolumeName));
-   ASSERT(dcr->VolumeName[0]);
+   ASSERT2(dcr->VolumeName[0], "Empty Volume name");
    rec->data_len = ser_length(rec->data);
    rec->FileIndex = dev->VolHdr.LabelType;
    Dmsg1(100, "LabelType=%d\n", dev->VolHdr.LabelType);
@@ -843,7 +854,7 @@ bool write_session_label(DCR *dcr, int label)
       rec->remainder);
 
    free_record(rec);
-   Dmsg2(150, "Leave write_session_label Block=%ud File=%ud\n",
+   Dmsg2(150, "Leave write_session_label Block=%u File=%u\n",
       dev->get_block_num(), dev->get_file());
    Leave(100);
    return true;
@@ -1107,13 +1118,74 @@ static void dump_session_label(DEV_RECORD *rec, const char *type)
    debug_level = dbl;
 }
 
-void dump_label_record(DEVICE *dev, DEV_RECORD *rec, int verbose)
+static int check_label(SESSION_LABEL *label)
+{
+   int  errors = 0;
+
+   if (label->JobId > 10000000 || label->JobId < 0) {
+      Pmsg0(-1, _("***** ERROR ****** : Found error with the JobId\n"));
+      errors++;
+   }
+
+   if (!errors) {
+      switch (label->JobLevel) {
+      case L_FULL:
+      case L_INCREMENTAL:
+      case L_DIFFERENTIAL:
+      case L_SINCE:
+      case L_VERIFY_CATALOG:
+      case L_VERIFY_INIT:
+      case L_VERIFY_VOLUME_TO_CATALOG:
+      case L_VERIFY_DISK_TO_CATALOG:
+      case L_VERIFY_DATA:
+      case L_BASE:
+      case L_NONE:
+      case L_VIRTUAL_FULL:
+         break;
+      default:
+         Pmsg0(-1, _("***** ERROR ****** : Found error with the JobLevel\n"));
+         errors++;
+      }
+   }
+   if (!errors) {
+      switch (label->JobType) {
+      case JT_BACKUP:
+            case JT_MIGRATED_JOB:
+      case JT_VERIFY:
+      case JT_RESTORE:
+      case JT_CONSOLE:
+      case JT_SYSTEM:
+      case JT_ADMIN:
+      case JT_ARCHIVE:
+      case JT_JOB_COPY:
+      case JT_COPY:
+      case JT_MIGRATE:
+      case JT_SCAN:
+               break;
+      default:
+         Pmsg0(-1, _("***** ERROR ****** : Found error with the JobType\n"));
+         errors++;
+      }
+   }
+   if (!errors) {
+      POOLMEM *err = get_pool_memory(PM_EMSG);
+      if (!is_name_valid(label->Job, &err)) {
+         Pmsg1(-1, _("***** ERROR ****** : Found error with the Job name %s\n"), err);
+         errors++;
+      }
+      free_pool_memory(err);
+   }
+   return errors;
+}
+
+int dump_label_record(DEVICE *dev, DEV_RECORD *rec, int verbose, bool check_err)
 {
    const char *type;
    int64_t dbl;
+   int errors = 0;
 
    if (rec->FileIndex == 0 && rec->VolSessionId == 0 && rec->VolSessionTime == 0) {
-      return;
+      return 0;
    }
    dbl = debug_level;
    debug_level = 1;
@@ -1147,10 +1219,9 @@ void dump_label_record(DEVICE *dev, DEV_RECORD *rec, int verbose)
          unser_volume_label(dev, rec);
          dump_volume_label(dev);
          break;
-      case SOS_LABEL:
-         dump_session_label(rec, type);
-         break;
+
       case EOS_LABEL:
+      case SOS_LABEL:
          dump_session_label(rec, type);
          break;
       case EOM_LABEL:
@@ -1178,6 +1249,9 @@ void dump_label_record(DEVICE *dev, DEV_RECORD *rec, int verbose)
             type, dev->file, dev->block_num, rec->VolSessionId, rec->VolSessionTime, label.JobId);
          Pmsg4(-1, _("   Job=%s Date=%s Level=%c Type=%c\n"),
             label.Job, dt, label.JobLevel, label.JobType);
+         if (check_err) {
+            errors += check_label(&label);
+         }
          break;
       case EOS_LABEL:
          char ed1[30], ed2[30];
@@ -1190,6 +1264,9 @@ void dump_label_record(DEVICE *dev, DEV_RECORD *rec, int verbose)
             edit_uint64_with_commas(label.JobFiles, ed1),
             edit_uint64_with_commas(label.JobBytes, ed2),
             label.JobErrors, (char)label.JobStatus);
+         if (check_err) {
+            errors += check_label(&label);
+         }
          break;
       case EOM_LABEL:
       case PRE_LABEL:
@@ -1204,4 +1281,5 @@ void dump_label_record(DEVICE *dev, DEV_RECORD *rec, int verbose)
       }
    }
    debug_level = dbl;
+   return errors;
 }
