@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *   Drive reservation functions for Storage Daemon
@@ -445,7 +448,8 @@ bool find_suitable_device_for_job(JCR *jcr, RCTX &rctx)
                }
                if (vol->dev->is_autochanger()) {
                   Dmsg1(dbglvl, "vol=%s is in changer\n", vol->vol_name);
-                  if (!is_vol_in_autochanger(rctx, vol) || !vol->dev->autoselect) {
+                  if (!is_vol_in_autochanger(rctx, vol) || !vol->dev->autoselect ||
+                     !vol->dev->enabled) {
                      continue;
                   }
                } else if (strcmp(device_name, vol->dev->device->hdr.name) != 0) {
@@ -603,8 +607,12 @@ static bool is_vol_in_autochanger(RCTX &rctx, VOLRES *vol)
 {
    AUTOCHANGER *changer = vol->dev->device->changer_res;
 
+   if (!changer) {
+      return false;
+   }
+
    /* Find resource, and make sure we were able to open it */
-   if (changer && strcmp(rctx.device_name, changer->hdr.name) == 0) {
+   if (strcmp(rctx.device_name, changer->hdr.name) == 0) {
       Dmsg1(dbglvl, "Found changer device %s\n", vol->dev->device->hdr.name);
       return true;
    }
@@ -648,6 +656,11 @@ static int reserve_device(RCTX &rctx)
               rctx.device_name);
       }
       return -1;  /* no use waiting */
+   } else if (!rctx.device->dev->enabled) {
+      Jmsg(rctx.jcr, M_WARNING, 0, _("\n"
+           "     Device \"%s\" requested by DIR is disabled.\n"),
+              rctx.device_name);
+      return -1;  /* no use waiting */
    }
 
    rctx.suitable_device = true;
