@@ -1,22 +1,25 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *   Volume management functions for Storage Daemon
  *
- *   Written by Kern Sibbald, MM
+ *   Kern Sibbald, MM
  *
  *   Split from reserve.c October 2008
  *
@@ -206,12 +209,13 @@ static void debug_list_volumes(const char *imsg)
 
    foreach_vol(vol) {
       if (vol->dev) {
-         Mmsg(msg, "List %s: %s in_use=%d swap=%d on %s device %s\n", imsg,
+         Mmsg(msg, "List %s: %s in_use=%d swap=%d slot=%d on %s device %s\n", imsg,
               vol->vol_name, vol->is_in_use(), vol->is_swapping(),
+              vol->get_slot(),
               vol->dev->print_type(), vol->dev->print_name());
       } else {
-         Mmsg(msg, "List %s: %s in_use=%d swap=%d no dev\n", imsg, vol->vol_name,
-              vol->is_in_use(), vol->is_swapping());
+         Mmsg(msg, "List %s: %s in_use=%d swap=%d slot=%d no dev\n", imsg, vol->vol_name,
+              vol->is_in_use(), vol->is_swapping(), vol->get_slot());
       }
       Dmsg1(dbglvl, "%s", msg.c_str());
    }
@@ -278,8 +282,8 @@ static VOLRES *new_vol_item(DCR *dcr, const char *VolumeName)
    vol->vol_name = bstrdup(VolumeName);
    if (dcr) {
       vol->dev = dcr->dev;
-      Dmsg3(dbglvl, "new Vol=%s at %p dev=%s\n",
-            VolumeName, vol->vol_name, vol->dev->print_name());
+      Dmsg4(dbglvl, "new Vol=%s slot=%d at %p dev=%s\n",
+            VolumeName, vol->get_slot(), vol->vol_name, vol->dev->print_name());
    }
    vol->init_mutex();
    vol->inc_use_count();
@@ -404,8 +408,8 @@ VOLRES *reserve_volume(DCR *dcr, const char *VolumeName)
        *  is not being used and is marked as not reserved.
        */
       if (strcmp(vol->vol_name, VolumeName) == 0) {
-         Dmsg2(dbglvl, "set reserved vol=%s dev=%s\n", VolumeName,
-               vol->dev->print_name());
+         Dmsg3(dbglvl, "set reserved vol=%s slot=%d dev=%s\n", VolumeName,
+               vol->get_slot(), vol->dev->print_name());
          goto get_out;                  /* Volume already on this device */
       } else {
          /* Don't release a volume if it was reserved by someone other than us */
@@ -420,7 +424,7 @@ VOLRES *reserve_volume(DCR *dcr, const char *VolumeName)
          Dmsg2(dbglvl, "reserve_vol free vol=%s at %p\n", vol->vol_name, vol->vol_name);
          /* If old Volume is still mounted, must unload it */
          if (strcmp(vol->vol_name, dev->VolHdr.VolumeName) == 0) {
-            Dmsg0(50, "set_unload\n");
+            Dmsg2(50, "set_unload vol=%s slot=%d\n", vol->vol_name, vol->get_slot());
             dev->set_unload();          /* have to unload current volume */
          }
          free_volume(dev);              /* Release old volume entry */
@@ -486,7 +490,8 @@ VOLRES *reserve_volume(DCR *dcr, const char *VolumeName)
             Dmsg3(dbglvl, "==== Swap vol=%s from dev=%s to %s\n",
                VolumeName, vol->dev->print_name(), dev->print_name());
             free_volume(dev);            /* free any volume attached to our drive */
-            Dmsg1(50, "set_unload dev=%s\n", dev->print_name());
+            Dmsg3(50, "set_unload vol=%s slot=%d dev=%s\n", vol->vol_name,
+               vol->get_slot(), dev->print_name());
             dev->set_unload();           /* Unload any volume that is on our drive */
             dcr->set_dev(vol->dev);      /* temp point to other dev */
             slot = get_autochanger_loaded_slot(dcr);  /* get slot on other drive */
@@ -503,26 +508,27 @@ VOLRES *reserve_volume(DCR *dcr, const char *VolumeName)
             if (dev) {
                Jmsg8(jcr, M_WARNING, 0, "Need volume for %s from other drive, "
                   "but swap not possible. Status: reader=%d writers=%d "
-                  "reserves=%d swap=%d vol=%s from dev=%s to %s\n", 
+                  "reserves=%d swap=%d vol=%s from dev=%s to %s\n",
                   dcr->is_writing()?"write":"read",
                   vol->dev->can_read(), vol->dev->num_writers,
                   vol->dev->num_reserved(), vol->is_swapping(),
                   VolumeName, vol->dev->print_name(), dev->print_name());
             }
             if (vol->is_swapping()) {
-               if (vol->dev && vol->dev->swap_dev && dev && dev->swap_dev) {
+               DEVICE *swapdev = dev->swap_dev;
+               if (vol && dev && swapdev) {
                   Mmsg3(jcr->errmsg, _("Volume %s is busy swapping from %s to %s\n"),
-                     vol->vol_name, dev->print_name(), dev->swap_dev->print_name());
+                     NPRT(vol->vol_name), dev->print_name(), swapdev->print_name());
                } else {
                   Mmsg1(jcr->errmsg, _("Volume %s is busy swapping.\n"),
-                     vol->vol_name);
+                     NPRT(vol->vol_name));
                }
             } else if (vol->dev) {
-               Mmsg2(jcr->errmsg, _("%s device %s is busy.\n"), 
+               Mmsg2(jcr->errmsg, _("%s device %s is busy.\n"),
                   vol->dev->print_type(), vol->dev->print_name());
             } else {
                Mmsg1(jcr->errmsg, _("Volume %s is busy swapping.\n"),
-                  vol->vol_name);
+                  NPRT(vol->vol_name));
             }
             debug_list_volumes("failed swap");
             vol = NULL;                  /* device busy */
@@ -683,7 +689,8 @@ bool volume_unused(DCR *dcr)
       return false;
    }
 
-   Dmsg1(dbglvl, "Clear in_use vol=%s\n", dev->vol->vol_name);
+   Dmsg2(dbglvl, "Clear in_use vol=%s slot=%d\n", dev->vol->vol_name,
+         dev->vol->get_slot());
    dev->vol->clear_in_use();
 
    if (dev->vol->is_swapping()) {
@@ -698,8 +705,9 @@ bool volume_unused(DCR *dcr)
     *  explicitly read in this drive. This allows the SD to remember
     *  where the tapes are or last were.
     */
-   Dmsg4(dbglvl, "set not reserved vol=%s writers=%d reserves=%d dev=%s\n",
-      dev->vol->vol_name, dev->num_writers, dev->num_reserved(), dev->print_name());
+   Dmsg5(dbglvl, "set not reserved vol=%s slot=%d writers=%d reserves=%d dev=%s\n",
+      dev->vol->vol_name, dev->vol->get_slot(), dev->num_writers, 
+      dev->num_reserved(), dev->print_name());
    if (dev->is_tape() || dev->is_autochanger()) {
       return true;
    } else {
@@ -729,12 +737,13 @@ bool free_volume(DEVICE *dev)
    }
    /* Don't free a volume while it is being swapped */
    if (!vol->is_swapping()) {
-      Dmsg1(dbglvl, "Clear in_use vol=%s\n", vol->vol_name);
+      Dmsg2(dbglvl, "Clear in_use vol=%s slot=%d\n", vol->vol_name, vol->get_slot());
       dev->vol = NULL;
       if (vol->is_writing()) {
          vol_list->remove(vol);
       }
-      Dmsg2(dbglvl, "Remove volume %s dev=%s\n", vol->vol_name, dev->print_name());
+      Dmsg3(dbglvl, "Remove volume %s slot=%d dev=%s\n", vol->vol_name, 
+         vol->get_slot(), dev->print_name());
       free_vol_item(vol);
       debug_list_volumes("free_volume");
    } else {
