@@ -5,9 +5,9 @@
 #define COMMUNITY 1      /* Define to create a Windows community binary */
 
 /* Note: there can be only *one* VERSION in this file */
-#define VERSION "7.4.3"
-#define BDATE   "18 June 2016"
-#define LSMDATE "18Jun16"
+#define VERSION "7.4.4"
+#define BDATE   "20 September 2016"
+#define LSMDATE "202Sep16"
 
 #define RELEASE 1   /* Use ONLY in rpms */
 
